UNIT mp3pls;

(* Playlisten-Verwaltung fr MP3-Player *)
INTERFACE

Const NetSyncDrive = 'W:\';
      MinFreeHdSpace = 2*1024*1024; (* min 2MB HD frei! *)
TYPE IndexRecord = RECORD
       Title: STRING[50];
       Artist:STRING[50];
       Comment:STRING[80];
       Path:  STRING[100];
       Quality: CHAR;
     END;
     IndexFile = FILE OF IndexRecord;
     GlobalIndexRec = RECORD
       Data: IndexRecord;
       Drive:BYTE;
     END;
     PLSIndexEntry = RECORD
       Title: STRING[50]; (* Playlistenname *)
       Path:  STRING[80];
     END;
     PLSEntry = RECORD
       Title: STRING[50]; (* erster Eintrag is der Name *)
       Artist:STRING[50]; (* im ersten Eintrag Artist='*' => gendert *)
     END;
     PLSIndicator = ARRAY[0..8191] OF BYTE;
     PPLSIndicator = ^PLSIndicator;

FUNCTION IntToStr(i: LONGINT): STRING;
FUNCTION FileExists(datei: string):BOOLEAN;
PROCEDURE ReScanDrives;
PROCEDURE ReScanDrive(num: BYTE);
PROCEDURE ImportIndex(DriveNo: BYTE);
PROCEDURE CreateGlobalIndex;
PROCEDURE CreateTextIndex(dn: STRING);
PROCEDURE GetID3Info(name: STRING; VAR title,artist,comment: STRING);
FUNCTION GetTrackPath(title,artist: STRING): STRING;
FUNCTION GetDiscPath(no: BYTE): STRING;
FUNCTION MakeCompareable(s: STRING): STRING;
FUNCTION PlaylistGetTrack(num: LONGINT; VAR title,artist: STRING): BOOLEAN;
FUNCTION PlaylistGetTrackCount: LONGINT;
PROCEDURE PlaylistAddTitle(title,artist: STRING);
PROCEDURE PlaylistRemoveTitle(title,artist: STRING);
PROCEDURE PlaylistSwap(i: LONGINT);
PROCEDURE CreateNewPlaylist;
FUNCTION PlaylistChanged: BOOLEAN;
PROCEDURE SavePlaylist(name: STRING);
PROCEDURE LoadPlaylist(dn: STRING);
PROCEDURE PlaylistDelete(dn: STRING);
FUNCTION PlaylistGetName: STRING;
PROCEDURE NetworkSync;
PROCEDURE CreateCompleteIndex(path,dn: STRING);
FUNCTION InPLSbyArtist(num: LONGINT): BOOLEAN;
FUNCTION InPLSbyTitle(num: LONGINT): BOOLEAN;

IMPLEMENTATION

USES DOS,crt,MP3UICOM;

VAR  PLSArtistIndicator: PPLSIndicator;
     PLSTitleIndicator: PPLSIndicator;

PROCEDURE PLSIndicatorADD(plsi: PPLSIndicator; num: WORD); FORWARD;
PROCEDURE PLSIndicatorDEL(plsi: PPLSIndicator; num: WORD); FORWARD;
PROCEDURE RecreatePLSIndicators; FORWARD;


FUNCTION GetTempStr(c: BYTE): STRING;
VAR s: STRING;
    zae: BYTE;
BEGIN
  s:='';
  FOR zae:=1 TO c DO
    s:=s+CHR(65+RANDOM(26));
  GetTempStr:=s;
END;

FUNCTION FileExists(datei: string):BOOLEAN;

VAR sr: SearchRec;

BEGIN
  FindFirst(datei,Archive,sr);
  IF DosError=0 THEN FileExists:=TRUE ELSE FileExists:=FALSE;
END;

FUNCTION IsDirectory(dn: string): BOOLEAN;
VAR f: FILE;
    at: WORD;
BEGIN
  ASSIGN(f,dn);
  GetFAttr(f,at);
  IsDirectory:=(at AND Directory)<>0;
END;

FUNCTION IntToStr(i: LONGINT): STRING;

VAR s: STRING[11];

BEGIN
  STR(i,s);
  IntToStr:=s;
END;

FUNCTION Upper(s: string): string;
VAR z: BYTE;
BEGIN
  FOR z:=1 to LENGTH(s) DO
    s[z]:=UpCase(s[z]);
  Upper:=s;
END;

FUNCTION GetDiscPath(no: BYTE): STRING;
(* Liefet Laufwerkspfad fr Platte no zurck *)
BEGIN
(*  IF no=0 THEN BEGIN
    GetDiscPath:='C:\MP3';
  END ELSE BEGIN*)
    GetDiscPath:=CHR(ORD('C')+no)+':';
(*  END;*)
END;

VAR a,b,c: IndexRecord;
    va,vb,vc: STRING[50];
    pe: BYTE;

PROCEDURE QuickSortTitle(VAR ndx: IndexFile; start,ende: LONGINT);
(* In Datei ndx von start bis ende Sortieren *)
VAR r,l: LONGINT;

BEGIN
  IF start>=ende THEN EXIT; (* Abbruchkriterium *)
  (* ein Mittleres (Pivot-)Element suchen *)
  SEEK(ndx,start);
  READ(ndx,a);
  SEEK(ndx,(start+ende) DIV 2);
  READ(ndx,b);
  SEEK(ndx,ende);
  READ(ndx,c);
  va:=MakeCompareable(a.title);
  vb:=MakeCompareable(b.title);
  vc:=MakeCompareable(c.title);
  pe:=1;
  IF (va>vb) AND (va>vc) THEN
    pe:=2
  ELSE IF (va<vb) AND (va<vc) THEN pe:=2;
  IF pe=2 THEN IF (vb>va) AND (vb>vc) THEN
    pe:=3
  ELSE IF (vb<va) AND (vb<vc) THEN pe:=3;
  CASE pe OF
    1: (* nix zu tun *);
    2: BEGIN
         SEEK(ndx,start); WRITE(ndx,b);
         SEEK(ndx,(start+ende) DIV 2); WRITE(ndx,a);
         a:=b;
       END;
    3: BEGIN
         SEEK(ndx,start); WRITE(ndx,c);
         SEEK(ndx,ende); WRITE(ndx,a);
         a:=c;
       END;
  END;
  (* Pivot-Element ist in a *)
  va:=MakeCompareable(a.title);
  r:=ende;
  l:=start+1;
  WHILE l<r DO BEGIN
    SEEK(ndx,l);
    READ(ndx,b);
    WHILE (MakeCompareable(b.title)<=va) AND (l<r) DO BEGIN
      READ(ndx,b);
      INC(l);
    END;
    (* b enthlt erstes "grer"-Element *)
    SEEK(ndx,r);
    READ(ndx,c);
    WHILE (MakeCompareable(c.title)>va) AND (l<r) DO BEGIN
      DEC(r);
      SEEK(ndx,r);
      READ(ndx,c);
    END;
    IF r<>l THEN BEGIN
      (* die beiden ELemente tauschen *)
      SEEK(ndx,l);
      WRITE(ndx,c);
      SEEK(ndx,r);
      WRITE(ndx,b);
    END;
  END;
  SEEK(ndx,r);
  READ(ndx,c);
  IF MakeCompareable(c.title)>va THEN BEGIN
    (* Element r=l gehrt zur "greren" Hlfte *)
    DEC(l);
  END;
  SEEK(ndx,start);
  READ(ndx,a);
  SEEK(ndx,l);
  READ(ndx,b);
  SEEK(ndx,start);
  WRITE(ndx,b);
  SEEK(ndx,l);
  WRITE(ndx,a);
  QuickSortTitle(ndx,start,l-1);
  QuickSortTitle(ndx,l+1,ende);
END;

PROCEDURE QuickSortArtist(VAR ndx: IndexFile; start,ende: LONGINT);
(* In Datei ndx von start bis ende Sortieren *)
VAR r,l: LONGINT;

BEGIN
  IF start>=ende THEN EXIT; (* Abbruchkriterium *)
  (* ein Mittleres (Pivot-)Element suchen *)
  SEEK(ndx,start);
  READ(ndx,a);
  SEEK(ndx,(start+ende) DIV 2);
  READ(ndx,b);
  SEEK(ndx,ende);
  READ(ndx,c);
  va:=MakeCompareable(a.artist);
  vb:=MakeCompareable(b.artist);
  vc:=MakeCompareable(c.artist);
  pe:=1;
  IF (va>vb) AND (va>vc) THEN
    pe:=2
  ELSE IF (va<vb) AND (va<vc) THEN pe:=2;
  IF pe=2 THEN IF (vb>va) AND (vb>vc) THEN
    pe:=3
  ELSE IF (vb<va) AND (vb<vc) THEN pe:=3;
  CASE pe OF
    1: (* nix zu tun *);
    2: BEGIN
         SEEK(ndx,start); WRITE(ndx,b);
         SEEK(ndx,(start+ende) DIV 2); WRITE(ndx,a);
         a:=b;
       END;
    3: BEGIN
         SEEK(ndx,start); WRITE(ndx,c);
         SEEK(ndx,ende); WRITE(ndx,a);
         a:=c;
       END;
  END;
  (* Pivot-Element ist in a *)
  va:=MakeCompareable(a.artist);
  r:=ende;
  l:=start+1;
  WHILE l<r DO BEGIN
    SEEK(ndx,l);
    READ(ndx,b);
    WHILE (MakeCompareable(b.artist)<=va) AND (l<r) DO BEGIN
      READ(ndx,b);
      INC(l);
    END;
    (* b enthlt erstes "grer"-Element *)
    SEEK(ndx,r);
    READ(ndx,c);
    WHILE (MakeCompareable(c.artist)>va) AND (l<r) DO BEGIN
      DEC(r);
      SEEK(ndx,r);
      READ(ndx,c);
    END;
    IF r<>l THEN BEGIN
      (* die beiden ELemente tauschen *)
      SEEK(ndx,l);
      WRITE(ndx,c);
      SEEK(ndx,r);
      WRITE(ndx,b);
    END;
  END;
  SEEK(ndx,r);
  READ(ndx,c);
  IF MakeCompareable(c.artist)>va THEN BEGIN
    (* Element r=l gehrt zur "greren" Hlfte *)
    DEC(l);
  END;
  SEEK(ndx,start);
  READ(ndx,a);
  SEEK(ndx,l);
  READ(ndx,b);
  SEEK(ndx,start);
  WRITE(ndx,b);
  SEEK(ndx,l);
  WRITE(ndx,a);
  QuickSortArtist(ndx,start,l-1);
  QuickSortArtist(ndx,l+1,ende);
END;


PROCEDURE CreateTitleSubIndex(DriveNo: BYTE);
(* erzeugt eine Subdatenbank, nach Titel sortiert *)
VAR ndx: IndexFile;
    idf: TEXT;
    ir,ir2:  IndexRecord;
    s:   STRING;
    swapped: BOOLEAN;
    count,zae: INTEGER;
    error: BOOLEAN;

BEGIN
  ASSIGN(idf,'C:\INDEX\CONT'+IntToSTr(DriveNo)+'.IDA');
  ASSIGN(ndx,'C:\INDEX\IDXT'+IntToStr(DriveNo)+'.IDX');
  RESET(idf);
  REWRITE(ndx);
  count:=0;
  WHILE NOT EOF(idf) DO BEGIN
    READLN(idf,s);
    IF s='--' THEN BEGIN
      READLN(idf,s); (* Titel *)
      error:=EOF(idf);
      IF LENGTH(s)>50 THEN s[0]:=#50;
      ir.Title:=s;
      READLN(idf,s); (* Interpret *)
      error:=error OR EOF(idf);
      IF LENGTH(s)>50 THEN s[0]:=#50;
      ir.Artist:=s;
      READLN(idf,s); (* Joliet-Pfad *)
      READLN(idf,s); (* ISO-Pfad *)
      error:=error OR EOF(idf);
      IF LENGTH(s)>100 THEN s[0]:=#100;
      ir.Path:=s;
      READLN(idf,s); (* Quality *)
      error:=error OR EOF(idf);
      ir.Quality:=s[1];
      READLN(idf,s); (* Bemerkungen *)
      IF LENGTH(s)>80 THEN s[0]:=#80;
      ir.Comment:=s;
      IF NOT error THEN BEGIN
        WRITE(ndx,ir);
        INC(count);
      END;
    END;
  END;
  CLOSE(idf); (* So, Index liegt als Datenbank vor *)
  (* Sortieren... *)
  (* Quicksort *)
  QuickSortTitle(ndx,0,FileSize(ndx)-1);
  CLOSE(ndx);
END;

PROCEDURE CreateArtistSubIndex(DriveNo: BYTE);
(* erzeugt eine Subdatenbank, nach Interpret sortiert *)
VAR ndx: IndexFile;
    idf: FILE OF IndexRecord;
    ir,ir2:  IndexRecord;
    swapped: BOOLEAN;
    count,zae: INTEGER;
    fz,al,ar: LONGINT;

BEGIN
  ASSIGN(idf,'C:\INDEX\IDXT'+IntToSTr(DriveNo)+'.IDX');
  ASSIGN(ndx,'C:\INDEX\IDXA'+IntToStr(DriveNo)+'.IDX');
  RESET(idf);
  REWRITE(ndx);
  count:=0;
  WHILE NOT EOF(idf) DO BEGIN
    READ(idf,ir);
    WRITE(ndx,ir);
    INC(count);
  END;
  CLOSE(idf);
  (* Quicksort nach Artist-Eintrgen *)
  QuickSortArtist(ndx,0,FileSize(ndx)-1);
  (* Jetzt noch Titel mit gleichem Interpreten rausfischen *)
  SEEK(ndx,0);
  IF FileSize(ndx)>0 THEN READ(ndx,ir);
  al:=0; ar:=0;
  FOR fz:=1 TO FileSize(ndx)-1 DO BEGIN
    SEEK(ndx,fz);
    READ(ndx,ir2);
    IF MakeCompareable(ir.artist)<>MakeCompareable(ir2.artist) THEN BEGIN
      QuickSortTitle(ndx,al,ar);
      al:=fz; ar:=fz;
      ir:=ir2;
    END ELSE BEGIN
      ar:=fz;
    END;
  END;
  QuickSortTitle(ndx,al,ar);
  CLOSE(ndx);
END;

PROCEDURE GetID3Info(name: STRING; VAR title,artist,comment: STRING);
(* ID3-Tags auslesen, wenn mglich *)
VAR mf: FILE;
    s: STRING[200];
    r: WORD;
    p: BYTE;
BEGIN
  FileMode:=0;
  ASSIGN(mf,name);
  RESET(mf,1);
  FileMode:=2;
  SEEK(mf,FileSize(mf)-200);
  BLOCKREAD(mf,s,200,r);
  CLOSE(mf);
  s[0]:=#199;
  p:=POS('TAG',s);
  IF p<>0 THEN BEGIN
    (* ID3-Tags vorhanden *)
    title:=COPY(s,p+3,30);
    artist:=COPY(s,p+33,30);
    comment:=COPY(s,p+97,30);
    IF POS(#00,title)<>0 THEN title:=COPY(title,1,POS(#00,title)-1);
    IF POS(#00,artist)<>0 THEN artist:=COPY(artist,1,POS(#00,artist)-1);
    IF POS(#00,comment)<>0 THEN comment:=COPY(comment,1,POS(#00,comment)-1);
    WHILE (LENGTH(title)>0) AND (title[LENGTH(title)]<'!') DO title[0]:=CHR(ORD(title[0])-1);
    WHILE (LENGTH(artist)>0) AND (artist[LENGTH(artist)]<'!') DO artist[0]:=CHR(ORD(artist[0])-1);
    WHILE (LENGTH(comment)>0) AND (comment[LENGTH(comment)]<'!') DO comment[0]:=CHR(ORD(comment[0])-1);
    IF title='' THEN title:='Unknown (ID3 tag empty)';
    IF artist='' THEN artist:='Unknown (ID3 tag empty)';
  END ELSE BEGIN
    (* Nur DOS-Name da *)
    title:=name;
    WHILE POS('\',title)<>0 DO BEGIN
      p:=POS('\',title);
      title:=COPY(title,p+1,LENGTH(title)-p);
    END;
    artist:='DOS/ISO 9660 Filesystem';
    comment:='';
  END;
END;

PROCEDURE CopyNdxFile(src,dest: STRING);
(* Textdatei kopieren *)
VAR sf,df: TEXT;
    s: STRING;
BEGIN
  ASSIGN(sf,src);
  ASSIGN(df,dest);
  FileMode:=0;
  RESET(sf);
  FileMode:=2;
  REWRITE(df);
  WHILE NOT EOF(sf) DO BEGIN
    READLN(sf,s);
    WRITELN(df,s);
  END;
  CLOSE(sf);
  CLOSE(df);
END;

PROCEDURE CreateNdx_ThisDir(VAR idf: TEXT; src: STRING);
(* Dateien in einem Directory durchsehen *)
VAR sr: SearchRec;
    fpath: STRING;
    id3_Title,id3_Artist,id3_Comment: STRING;

BEGIN
  FindFirst(src+'*.MP3',Archive+ReadOnly,sr);
  WHILE DosError=0 DO BEGIN
    (* sr enthlt Datei-Info *)
    GetID3Info(src+sr.name,id3_Title,id3_Artist,id3_comment);
    (* Indexdatei erweitern *)
    fpath:=src+sr.name; (* ist voll qualifiziert, d.h. mit Laufwerk *)
    fpath:=COPY(fpath,3,LENGTH(fpath)-2);
    WRITELN(idf,'--');
    WRITELN(idf,id3_Title);
    WRITELN(idf,id3_Artist);
    WRITELN(idf,''); (* Joliet-Pfad unbekannt *)
    WRITELN(idf,fpath); (* ISO bzw. DOS-Name *)
    WRITELN(idf,'C'); (* unbekannt, nehmen wir mal C an *)
    WRITELN(idf,id3_comment);
    (* und weiter gehts *)
    FindNext(sr);
  END;
END;

PROCEDURE CreateNdx_SubDirs(VAR idf: TEXT; src: STRING);
(* Unterverzeichnisse absuchen *)
VAR sr: SearchRec;

BEGIN
  FindFirst(src+'*.*',Directory,sr);
  WHILE DosError=0 DO BEGIN
    (* sr enthlt gltiges Dir *)
    IF (sr.name<>'.') AND (sr.name<>'..') AND ((sr.Attr AND Directory)<>0) THEN BEGIN
      CreateNdx_ThisDir(idf,src+sr.name+'\');
      CreateNdx_SubDirs(idf,src+sr.name+'\');
    END;
    FindNext(sr);
  END;
END;

PROCEDURE CreateNdxFile(src,dest: STRING);
(* Indexdatei dest aufgrund von Dateistruktur ab src erstellen *)
(* d.h. alle Dateien durchhangeln... *)
VAR idf: TEXT;
    sr: SearchRec;
    TheVolume: STRING;

BEGIN
  (* Mglicherweise Defautl-File vorhanden ? *)
  FindFirst(src+'*.*',VolumeId,sr);
  TheVolume:=sr.name; (* VolumeId des Datentrgers *)
  IF (DosError=0) AND FileExists('C:\INDEX\DEFAULT\'+TheVolume) THEN BEGIN
    (* Wir haben eine Default-Datei *)
    CopyNdxFile('C:\INDEX\DEFAULT\'+TheVolume,dest);
  END ELSE BEGIN
    ASSIGN(idf,dest);
    REWRITE(idf);
    WRITELN(idf,'# This file was automatically generated from ID3-Tags and DOS-Filenames');
    CreateNdx_ThisDir(idf,src);
    CreateNdx_SubDirs(idf,src);
    CLOSE(idf);
  END;
END;

PROCEDURE ImportIndex(DriveNo: BYTE);

(* DriveNo=0 meint Lokale Daten in C:\MP3\
           1..6 meint Slot 1..6 vom CD-Wechsler Laufwerk D:
           7..255 reserviert fr knftige Anwendung *)
VAR IndexPath: STRING;
BEGIN
  IndexPath:=GetDiscPath(DriveNo)+'\'; (* Slot whlen *)
  (* IndexPath enthlt Pfad zu Indexdatei *)
  IF FileExists(IndexPath+'CONTENTS.IDA') THEN BEGIN
    (* klasse, CD mit Indexdatei *)
    CopyNdxFile(IndexPath+'CONTENTS.IDA','C:\INDEX\CONT'+IntToSTr(DriveNo)+'.IDA');
  END ELSE BEGIN
    (* Indexdatei selber herstellen *)
    CreateNdxFile(IndexPath,'C:\INDEX\CONT'+IntToSTr(DriveNo)+'.IDA');
  END;
  (* Jetzt steht Indexfile lokal zur Verfgung *)
  CreateTitleSubIndex(DriveNo);
  CreateArtistSubIndex(DriveNo);
END;

FUNCTION MakeCompareable(s: STRING): STRING;
VAR tmp: STRING;
    zae: BYTE;
BEGIN
  tmp:='';
  FOR zae:=1 to length(s) DO BEGIN
    CASE s[zae] OF
      #192..#198,#224..#230: tmp:=tmp+'A';
      #199,#231: tmp:=tmp+'C';
      #200..#203,#232..#235: tmp:=tmp+'E';
      #204..#207,#236..#239: tmp:=tmp+'I';
      #208,#240: tmp:=tmp+'D';
      #209,#241: tmp:=tmp+'N';
      #210..#214,#242..#246: tmp:=tmp+'O';
      #217..#220,#249..#252: tmp:=tmp+'U';
      #221,#253,#255: tmp:=tmp+'Y';
      #223: tmp:=tmp+'ss';
      'A'..'Z','a'..'z','0'..'9','&': tmp:=tmp+UpCase(s[zae]);
      (* Alle anderen Zeichen ignorieren *)
    END;
  END;
  MakeCompareable:=tmp;
END;

PROCEDURE CreateGlobalTitleIndex;
(* neuen Gesamtindex Titel erstellen aus den Dateien IDXT?.IDX *)
(* im Prinzip einfaches MergeSort *)
(* Maximal 10 Subindizes mglich *)
VAR fh: ARRAY[0..9] OF FILE OF IndexRecord;
    fno: ARRAY[0..9] OF BYTE;
    filecount: BYTE;
    zae: BYTE;
    entrycount: LONGINT;
    entry: ARRAY[0..9] OF IndexRecord;
    entryvalid: ARRAY[0..9] OF BOOLEAN;
    ndx: FILE OF GlobalIndexRec; (* Ausgabedatei *)
    globalentry: GlobalIndexRec;
    oldglobalentry: GlobalIndexRec;
    efound,swapped: BOOLEAN;
    myentry: BYTE;
BEGIN
  ASSIGN(ndx,'C:\INDEX\TITLE.IDX');
  REWRITE(ndx);
  filecount:=0;
  entrycount:=0;
  oldglobalentry.data.title:='';
  oldglobalentry.data.artist:='';
  oldglobalentry.data.quality:='Z';
  FOR zae:=0 to 9 DO BEGIN
    IF FileExists('C:\INDEX\IDXT'+IntToStr(zae)+'.IDX') THEN BEGIN
      ASSIGN(fh[filecount],'C:\INDEX\IDXT'+IntToStr(zae)+'.IDX');
      RESET(fh[filecount]);
      entrycount:=entrycount+FileSize(fh[filecount]);
      fno[filecount]:=zae;
      INC(filecount);
    END;
    entryvalid[zae]:=FALSE; (* kein Gltiger Datensatz fr dieses File geladen *)
  END;
  (* Alle Dateien geffnet, jetzt zusammensuchen *)
  IF filecount>0 THEN BEGIN
    REPEAT
      (* Datensatzbuffer auffllen *)
      FOR zae:=0 TO filecount-1 DO BEGIN
        IF NOT entryvalid[zae] THEN IF NOT EOF(fh[zae]) THEN BEGIN
          READ(fh[zae],entry[zae]);
          entryvalid[zae]:=TRUE;
        END;
      END;
      (* ersten vorhandenen Eintrag suchen *)
      efound:=FALSE;
      myentry:=0;
      WHILE (NOT efound) AND (myentry<filecount) DO BEGIN
        IF NOT entryvalid[myentry] THEN INC(myentry) ELSE efound:=TRUE;
      END;
      IF efound THEN BEGIN
        (* es gibt zumindest noch Eintrge *)
        REPEAT
          swapped:=FALSE;
          (* kleinsten Eintrag suchen *)
          FOR zae:=0 TO filecount-1 DO BEGIN
            IF entryvalid[zae] THEN
              IF MakeCompareable(entry[zae].Title)<MakeCompareable(entry[myentry].Title) THEN BEGIN
                myentry:=zae;
                swapped:=TRUE;
              END;
          END;
        UNTIL NOT swapped;
        (* myentry zeigt jetzt auf kleinsten Eintrag *)
        globalentry.data:=entry[myentry];
        globalentry.drive:=fno[myentry];
        (* Doubletten rausfischen *)
        IF (MakeCompareable(globalentry.data.title)=MakeCompareable(oldglobalentry.data.title)) AND
           (MakeCompareable(globalentry.data.artist)=MakeCompareable(oldglobalentry.data.artist)) THEN BEGIN
          (* Titel, Interpret gleich *)
          IF UpCase(globalentry.data.quality)<UpCase(oldglobalentry.data.quality) THEN BEGIN
            (* neuer Eintrag besser in Qualitt *)
            SEEK(ndx,FilePos(ndx)-1);
            Write(ndx,globalentry);
          END;
        END ELSE BEGIN
          Write(ndx,globalentry);
        END;
        oldglobalentry:=globalentry;
        entryvalid[myentry]:=FALSE;
      END;
    UNTIL NOT efound; (* nix mehr da... *)
    FOR zae:=0 TO filecount-1 DO
      Close(fh[zae]);
  END;
  Close(ndx);
END;
PROCEDURE CreateGlobalArtistIndex;
(* neuen Gesamtindex Titel erstellen aus den Dateien IDXA?.IDX *)
(* im Prinzip einfaches MergeSort *)
(* Maximal 10 Subindizes mglich *)
VAR fh: ARRAY[0..9] OF FILE OF IndexRecord;
    fno: ARRAY[0..9] OF BYTE;
    filecount: BYTE;
    zae: BYTE;
    entrycount: LONGINT;
    entry: ARRAY[0..9] OF IndexRecord;
    entryvalid: ARRAY[0..9] OF BOOLEAN;
    ndx: FILE OF GlobalIndexRec; (* Ausgabedatei *)
    globalentry,oldglobalentry: GlobalIndexRec;
    efound,swapped: BOOLEAN;
    myentry: BYTE;
BEGIN
  ASSIGN(ndx,'C:\INDEX\ARTIST.IDX');
  REWRITE(ndx);
  oldglobalentry.data.title:='';
  oldglobalentry.data.artist:='';
  oldglobalentry.data.quality:='Z';
  filecount:=0;
  entrycount:=0;
  FOR zae:=0 to 9 DO BEGIN
    IF FileExists('C:\INDEX\IDXA'+IntToStr(zae)+'.IDX') THEN BEGIN
      ASSIGN(fh[filecount],'C:\INDEX\IDXA'+IntToStr(zae)+'.IDX');
      RESET(fh[filecount]);
      entrycount:=entrycount+FileSize(fh[filecount]);
      fno[filecount]:=zae;
      INC(filecount);
    END;
    entryvalid[zae]:=FALSE; (* kein Gltiger Datensatz fr dieses File geladen *)
  END;
  (* Alle Dateien geffnet, jetzt zusammensuchen *)
  IF filecount>0 THEN BEGIN
    REPEAT
      (* Datensatzbuffer auffllen *)
      FOR zae:=0 TO filecount-1 DO BEGIN
        IF NOT entryvalid[zae] THEN IF NOT EOF(fh[zae]) THEN BEGIN
          READ(fh[zae],entry[zae]);
          entryvalid[zae]:=TRUE;
        END;
      END;
      (* ersten vorhandenen Eintrag suchen *)
      efound:=FALSE;
      myentry:=0;
      WHILE (NOT efound) AND (myentry<filecount) DO BEGIN
        IF NOT entryvalid[myentry] THEN INC(myentry) ELSE efound:=TRUE;
      END;
      IF efound THEN BEGIN
        (* es gibt zumindest noch Eintrge *)
        REPEAT
          swapped:=FALSE;
          (* kleinsten Eintrag suchen *)
          FOR zae:=0 TO filecount-1 DO BEGIN
            IF entryvalid[zae] THEN
              IF MakeCompareable(entry[zae].Artist)<=MakeCompareable(entry[myentry].Artist) THEN BEGIN
                IF MakeCompareable(entry[zae].Artist)<MakeCompareable(entry[myentry].Artist) THEN BEGIN
                  myentry:=zae;
                  swapped:=TRUE;
                END ELSE BEGIN
                  IF MakeCompareable(entry[zae].Title)<MakeCompareable(entry[myentry].Title) THEN BEGIN
                    myentry:=zae;
                    swapped:=TRUE;
                  END;
                END;
              END;
          END;
        UNTIL NOT swapped;
        (* myentry zeigt jetzt auf kleinsen Eintrag *)
        globalentry.data:=entry[myentry];
        globalentry.drive:=fno[myentry];
        (* Doubletten rausfischen *)
        IF (MakeCompareable(globalentry.data.title)=MakeCompareable(oldglobalentry.data.title)) AND
           (MakeCompareable(globalentry.data.artist)=MakeCompareable(oldglobalentry.data.artist)) THEN BEGIN
          (* Titel, Interpret gleich *)
          IF UpCase(globalentry.data.quality)<UpCase(oldglobalentry.data.quality) THEN BEGIN
            (* neuer Eintrag besser in Qualitt *)
            SEEK(ndx,FilePos(ndx)-1);
            Write(ndx,globalentry);
          END;
        END ELSE BEGIN
          Write(ndx,globalentry);
        END;
        oldglobalentry:=globalentry;
        entryvalid[myentry]:=FALSE;
      END;
    UNTIL NOT efound; (* nix mehr da... *)
    FOR zae:=0 TO filecount-1 DO
      Close(fh[zae]);
  END;
  Close(ndx);
END;

PROCEDURE CreateTextIndex(dn: STRING);
(* Den Artist-Index als Textfile erstellen *)
VAR ndx: FILE OF GlobalIndexRec; (* Ausgabedatei *)
    globalentry: GlobalIndexRec;
    ti: TEXT;
    t,a,b,q: string;
BEGIN
  ASSIGN(ndx,'C:\INDEX\ARTIST.IDX');
  RESET(ndx);
  ASSIGN(ti,dn);
  REWRITE(ti);
  WRITELN(ti,'Gesamtindex der vorhandenen Titel:');
  WRITELN(ti,'==================================');
  WRITELN(ti,'');
  WRITELN(ti,'Qualittsstufen:');
  WRITELN(ti,' A: von CD, Datenrate > 128kBit/s');
  WRITELN(ti,' B: von CD, Datenrate = 128kBit/s');
  WRITELN(ti,' C: von CD, Datenrate 112kBit/s oder Layer II codiert');
  WRITELN(ti,' D: z.B. aus Radio/TV; sehr gute Qualitt oder < 44kHz');
  WRITELN(ti,' E: aus Radio/TV; gute Qualitt');
  WRITELN(ti,' F: aus Radio/TV; befriedigende Qualitt');
  WRITELN(ti,' G: z.B. von Cassette; mige Qualitt');
  WRITELN(ti,' H: geringe Qualitt');
  WRITELN(ti,'');
  WRITELN(ti,FileSize(ndx),' Titel insgesamt.');
  WRITELN(ti,'');
  WRITELN(ti,'Interpret                     Titel                         Qly Bemerkungen');
  WRITELN(ti,'-------------------------------------------------------------------------------');
  WHILE NOT EOF(ndx) DO BEGIN
    READ(ndx,globalentry);
    t:=globalentry.data.title;
    a:=globalentry.data.artist;
    b:=globalentry.data.comment;
    q:=' '+globalentry.data.quality+'  ';
    IF LENGTH(t)>29 THEN t:=COPY(t,1,29);
    WHILE LENGTH(t)<30 DO t:=t+' ';
    IF LENGTH(a)>29 THEN a:=COPY(a,1,29);
    WHILE LENGTH(a)<30 DO a:=a+' ';
    IF LENGTH(b)>15 THEN b:=COPY(b,1,15);
    WRITELN(ti,a+t+q+b);
  END;
  CLOSE(ti);
  CLOSE(ndx);
END;

PROCEDURE CreateGlobalIndex;
(* Haupt-Indexdateien erzeugen *)
BEGIN
  CreateGlobalTitleIndex;
  CreateGlobalArtistIndex;
END;

PROCEDURE ReScanDrives;
(* Alle Laufwerke absuchen und neuen Index erstellen *)
VAR zae: BYTE;
BEGIN
  UI_CLS;
  UI_SetColors(ui_clWhite,ui_clBlack);
  FOR zae:=0 TO 9 DO BEGIN
    UI_GotoXY(1,zae+1);
    UI_Write('Durchsuche Laufwerk '+IntToStr(zae+1));
    IF GetDiscPath(zae)<>'' THEN ImportIndex(zae);
  END;
  UI_GotoXY(1,11);
  UI_Write('Index wird erstellt...');
  CreateGlobalIndex;
  ReCreatePLSIndicators;
END;

PROCEDURE ReScanDrive(num: BYTE);
(* Laufwerk absuchen und neuen Index erstellen *)
BEGIN
  UI_CLS;
  UI_SetColors(ui_clWhite,ui_clBlack);
  UI_GotoXY(1,2);
  UI_Write('Durchsuche Laufwerk '+IntToStr(num+1));
  IF GetDiscPath(num)<>'' THEN ImportIndex(num);
  UI_GotoXY(1,4);
  UI_Write('Index wird erstellt...');
  CreateGlobalIndex;
  ReCreatePLSIndicators;
END;

FUNCTION GetTrackPath(title,artist: STRING): STRING;
(* Sucht zum bergebenen Titel den Dateinamen *)
VAR rec: GlobalIndexRec;
    ndx: FILE OF GlobalIndexRec;
    l,r: LONGINT;
    tmpart,tmptit: STRING;
    tmp: STRING;
BEGIN
  tmp:='';
  title:=MakeCompareable(title);
  artist:=MakeCompareable(artist);
  ASSIGN(ndx,'C:\INDEX\ARTIST.IDX');
  RESET(ndx);
  l:=0;
  r:=FileSize(ndx)-1;
  IF console THEN BEGIN
    Gotoxy(1,24);
    Write('Suche nach '+artist+'-'+title);
  END;
  REPEAT
    SEEK(ndx,(l+r) DIV 2);
    READ(ndx,rec);
    tmpart:=MakeCompareable(rec.data.artist);
    tmptit:=MakeCompareable(rec.data.title);
    IF console THEN BEGIN
      Gotoxy(1,25);
      Write('Suche an Pos. ',(l+r) DIV 2,': '+tmpart+'-'+tmptit);
      readkey;
    END;
    IF (tmpart>artist) OR
       ((tmpart=artist) AND (tmptit>title)) THEN BEGIN
      (* gelesener Datensatz grer als gesuchter => linke Hlfte *)
      r:=((l+r) DIV 2)-1;
    END ELSE IF (tmpart=artist) AND (tmptit=title) THEN BEGIN
      (* gefunden *)
      tmp:=GetDiscPath(rec.drive)+rec.data.path;
    END ELSE BEGIN
      (* gelesener Datensatz kleiner als gesuchter => rechte Hlfte *)
      l:=((l+r) DIV 2)+1;
    END;
  UNTIL (tmp<>'') OR (l>r);
  CLOSE(ndx);
  GetTrackPath:=tmp;
END;

FUNCTION GetTrackArtistIndexNum(title,artist: STRING): LONGINT;
(* Sucht zum bergebenen Titel die Index-Nummer im Artist-Index *)
VAR rec: GlobalIndexRec;
    ndx: FILE OF GlobalIndexRec;
    l,r: LONGINT;
    tmpart,tmptit: STRING;
    tmp: LONGINT;
BEGIN
  tmp:=-1;
  title:=MakeCompareable(title);
  artist:=MakeCompareable(artist);
  ASSIGN(ndx,'C:\INDEX\ARTIST.IDX');
  RESET(ndx);
  l:=0;
  r:=FileSize(ndx)-1;
  REPEAT
    SEEK(ndx,(l+r) DIV 2);
    READ(ndx,rec);
    tmpart:=MakeCompareable(rec.data.artist);
    tmptit:=MakeCompareable(rec.data.title);
    IF (tmpart>artist) OR
       ((tmpart=artist) AND (tmptit>title)) THEN BEGIN
      (* gelesener Datensatz grer als gesuchter => linke Hlfte *)
      r:=((l+r) DIV 2)-1;
    END ELSE IF (tmpart=artist) AND (tmptit=title) THEN BEGIN
      (* gefunden *)
      tmp:=FilePos(ndx)-1;
    END ELSE BEGIN
      (* gelesener Datensatz kleiner als gesuchter => rechte Hlfte *)
      l:=((l+r) DIV 2)+1;
    END;
  UNTIL (tmp<>-1) OR (l>r);
  CLOSE(ndx);
  GetTrackArtistIndexNum:=tmp;
END;

FUNCTION GetTrackTitleIndexNum(title,artist: STRING): LONGINT;
(* Sucht zum bergebenen Titel die Index-Nummer im Titel-Index *)
VAR rec: GlobalIndexRec;
    ndx: FILE OF GlobalIndexRec;
    l,r: LONGINT;
    tmpart,tmptit: STRING;
    tmp: LONGINT;
BEGIN
  tmp:=-1;
  title:=MakeCompareable(title);
  artist:=MakeCompareable(artist);
  ASSIGN(ndx,'C:\INDEX\TITLE.IDX');
  RESET(ndx);
  l:=0;
  r:=FileSize(ndx)-1;
  REPEAT
    SEEK(ndx,(l+r) DIV 2);
    READ(ndx,rec);
    tmpart:=MakeCompareable(rec.data.artist);
    tmptit:=MakeCompareable(rec.data.title);
    IF tmptit>title THEN BEGIN
      (* gelesener Datensatz grer als gesuchter => linke Hlfte *)
      r:=((l+r) DIV 2)-1;
    END ELSE IF (tmptit=title) THEN BEGIN
      (* Titlebereich gefunden *)
      tmp:=FilePos(ndx)-1;
      WHILE (tmptit=title) AND (tmp>0) DO BEGIN
        DEC(tmp);
        SEEK(ndx,tmp);
        READ(ndx,rec);
        tmptit:=MakeCompareable(rec.data.title);
      END;
      (* Bereichsanfang... *)
      IF (tmptit<>title) THEN INC(tmp);
      REPEAT
        SEEK(ndx,tmp);
        READ(ndx,rec);
        tmpart:=MakeCompareable(rec.data.artist);
        tmptit:=MakeCompareable(rec.data.title);
      UNTIL (tmpart=artist) OR (tmptit<>title) OR EOF(ndx);
      IF (tmpart=artist) AND (tmptit=title) THEN
        tmp:=FilePos(ndx)-1
      ELSE
        tmp:=-1;
    END ELSE BEGIN
      (* gelesener Datensatz kleiner als gesuchter => rechte Hlfte *)
      l:=((l+r) DIV 2)+1;
    END;
  UNTIL (tmp<>-1) OR (l>r);
  CLOSE(ndx);
  GetTrackTitleIndexNum:=tmp;
END;


PROCEDURE CreatePlaylistIndex;
(* Durchsucht Verzeichnis C:\INDEX\PLAYLIST\ nach *.PLS *)
(* erstellt Indexfile C:\INDEX\PLAYLIST.IDX *)
VAR pli: FILE OF PLSIndexEntry;
    entry2,entry: PLSIndexEntry;
    pf: FILE OF PLSEntry;
    rec: PLSEntry;
    sr: SearchRec;
    plcount,zae: LONGINT;
    swapped: BOOLEAN;
BEGIN
  ASSIGN(pli,'C:\INDEX\PLAYLIST.IDX');
  REWRITE(pli);
  plcount:=0;
  FindFirst('C:\INDEX\PLAYLIST\*.PLS',Archive,sr);
  WHILE DosError=0 DO BEGIN
    ASSIGN(pf,'C:\INDEX\PLAYLIST\'+sr.name);
    RESET(pf);
    READ(pf,rec);
    CLOSE(pf); (* erster Datensatz in Playliste ist der Name *)
    IF LENGTH(rec.title)>ui_ColCount THEN rec.title:=COPY(rec.title,1,ui_ColCount);
    entry.title:=rec.title;
    entry.path:='C:\INDEX\PLAYLIST\'+sr.name;
    WRITE(pli,entry);
    INC(plcount);
    FindNext(sr);
  END;
  (* jetzt noch Sortieren *)
  (* Bubblesort, Shame on me *)
  REPEAT
    swapped:=FALSE;
    FOR zae:=0 TO plcount-2 DO BEGIN
      SEEK(pli,zae);
      READ(pli,entry);
      READ(pli,entry2);
      IF MakeCompareable(entry.title)>MakeCompareable(entry2.title) THEN BEGIN
        swapped:=TRUE;
        SEEK(pli,zae);
        WRITE(pli,entry2);
        WRITE(pli,entry);
      END;
    END;
  UNTIL NOT swapped;
  (* fertig *)
  CLOSE(pli);
END;

PROCEDURE CreateNewPlaylist;
(* Playliste CURRENT lschen *)
VAR pls: FILE OF PLSEntry;
    entry: PLSEntry;
BEGIN
  ASSIGN(pls,'C:\INDEX\CURRENT.PLS');
  REWRITE(pls);
  entry.title:=''; entry.artist:='';
  WRITE(pls,entry);
  CLOSE(pls);
  ReCreatePLSIndicators;
END;

FUNCTION PlaylistChanged: BOOLEAN;
(* TRUE, wenn aktuelle PLS gendert wurde *)
VAR pls: FILE OF PLSEntry;
    entry: PLSEntry;
BEGIN
  ASSIGN(pls,'C:\INDEX\CURRENT.PLS');
  RESET(pls);
  READ(pls,entry);
  CLOSE(pls);
  PlaylistChanged:=(entry.artist='*');
END;

PROCEDURE LoadPlaylist(dn: STRING);
(* Playliste zur aktuellen machen *)
VAR pls,cpls: FILE OF PLSEntry;
    entry: PLSEntry;
BEGIN
  ASSIGN(pls,dn);
  ASSIGN(cpls,'C:\INDEX\CURRENT.PLS');
  RESET(pls);
  REWRITE(cpls);
  WHILE NOT EOF(pls) DO BEGIN
    READ(pls,entry);
    WRITE(cpls,entry);
  END;
  CLOSE(pls);
  CLOSE(cpls);
  ReCreatePLSIndicators;
END;

PROCEDURE SavePlaylist(name: STRING);
(* aktuelle Playliste sichern; name mu nur dann
   einen Wert haben, wenn CURRENT.PLS noch keinen Namen hat *)
VAR pls,cpls: FILE OF PLSEntry;
    pif: FILE OF PLSIndexEntry;
    pie: PLSIndexEntry;
    entry: PLSEntry;
    dn: STRING;
BEGIN
  (* Gibs den Namen schonmal? *)
  ASSIGN(cpls,'C:\INDEX\CURRENT.PLS');
  RESET(cpls);
  READ(cpls,entry);
  IF name='' THEN name:=entry.title ELSE entry.title:=name;
  entry.artist:='';
  dn:='';
  IF FileExists('C:\INDEX\PLAYLIST.IDX') THEN BEGIN
    ASSIGN(pif,'C:\INDEX\PLAYLIST.IDX');
    RESET(pif);
    WHILE NOT EOF(pif) DO BEGIN
      READ(pif,pie);
      IF pie.title=name THEN dn:=pie.path;
    END;
    CLOSE(pif);
  END;
  IF dn='' THEN BEGIN
    (* Temporren Dateinamen finden *)
    REPEAT
      dn:='C:\INDEX\PLAYLIST\'+GetTempStr(8)+'.PLS'
    UNTIL NOT FileExists(dn);
  END;
  ASSIGN(pls,dn);
  REWRITE(pls);
  WRITE(pls,entry);
  WHILE NOT EOF(cpls) DO BEGIN
    READ(cpls,entry);
    WRITE(pls,entry);
  END;
  CLOSE(pls);
  CLOSE(cpls);
  CreatePlaylistIndex;
END;

PROCEDURE PlaylistDelete(dn: STRING);
VAR pls: FILE OF PLSEntry;
    entry1,entry2: PLSEntry;
BEGIN
  ASSIGN(pls,dn);
  IF FIleExists(dn) THEN BEGIN
    RESET(pls);
    READ(pls,entry1);
    CLOSE(pls);
    Erase(pls);
    ASSIGN(pls,'C:\INDEX\CURRENT.PLS');
    RESET(pls);
    READ(pls,entry2);
    IF entry1.title=entry2.title THEN BEGIN
      entry2.artist:='*';
      SEEK(pls,0);
      WRITE(pls,entry2);
    END;
    CLOSE(pls);
  END;
  CreatePlaylistIndex;
END;

PROCEDURE PlaylistAddTitle(title,artist: STRING);
(* Der aktuellen Playliste einen Titel zufgen *)
VAR cpls: FILE OF PLSEntry;
    entry: PLSEntry;
    found: BOOLEAN;
    tmp: LONGINT;
BEGIN
  ASSIGN(cpls,'C:\INDEX\CURRENT.PLS');
  RESET(cpls);
  SEEK(cpls,1);
  found:=FALSE;
  (* Suchen, ob der Titel schon drin ist *)
  WHILE NOT EOF(cpls) DO BEGIN
    READ(cpls,entry);
    IF (MakeCompareable(entry.title)=MakeCompareable(title)) AND
       (MakeCompareable(entry.artist)=MakeCompareable(artist)) THEN BEGIN
      found:=TRUE;
      SEEK(cpls,FILESIZE(cpls));
    END;
  END;
  IF NOT found THEN BEGIN
    entry.title:=title;
    entry.artist:=artist;
    WRITE(cpls,entry);
    SEEK(cpls,0);
    READ(cpls,entry);
    SEEK(cpls,0);
    entry.artist:='*'; (* gendert *)
    WRITE(cpls,entry);
    (* Titel zu Indikator-Liste zufgen *)
    tmp:=GetTrackArtistIndexNum(title,artist);
    IF tmp>=0 THEN PLSIndicatorADD(PLSArtistIndicator,tmp);
    (* Dito fr Titel !!!!*)
    tmp:=GetTrackTitleIndexNum(title,artist);
    IF tmp>=0 THEN PLSIndicatorADD(PLSTitleIndicator,tmp);
  END;
  CLOSE(cpls);
END;

PROCEDURE PlaylistRemoveTitle(title,artist: STRING);
(* Titel aus aktueller Playliste entfernen *)
VAR cpls: FILE OF PLSEntry;
    entry: PLSEntry;
    found: BOOLEAN;
    fp,zae,tmp: LONGINT;
BEGIN
  ASSIGN(cpls,'C:\INDEX\CURRENT.PLS');
  RESET(cpls);
  SEEK(cpls,1);
  found:=FALSE;
  (* Suchen, ob der Titel schon drin ist *)
  WHILE NOT EOF(cpls) DO BEGIN
    READ(cpls,entry);
    IF (MakeCompareable(entry.title)=MakeCompareable(title)) AND
       (MakeCompareable(entry.artist)=MakeCompareable(artist)) THEN BEGIN
      found:=TRUE;
      fp:=FilePos(cpls);
      SEEK(cpls,FileSize(cpls));
    END;
  END;
  IF found THEN BEGIN
    (* Titel entfernen *)
    FOR zae:=fp-1 TO FileSize(cpls)-2 DO BEGIN
      SEEK(cpls,zae+1);
      READ(cpls,entry);
      SEEK(cpls,zae);
      WRITE(cpls,entry);
    END;
    SEEK(cpls,0);
    READ(cpls,entry);
    entry.artist:='*'; (* gendert *)
    SEEK(cpls,0);
    WRITE(cpls,entry);
    SEEK(cpls,FileSize(cpls)-1);
    Truncate(cpls);
    (* Titel aus Indikator-Liste Lschen *)
    tmp:=GetTrackArtistIndexNum(title,artist);
    IF tmp>=0 THEN PLSIndicatorDEL(PLSArtistIndicator,tmp);
    (* Dito fr Titel !!!!*)
    tmp:=GetTrackTitleIndexNum(title,artist);
    IF tmp>=0 THEN PLSIndicatorDEL(PLSTitleIndicator,tmp);
 END;
  CLOSE(cpls);
END;

PROCEDURE PlaylistSwap(i: LONGINT);
(* Title in Playliste tauschen *)
VAR cpls: FILE OF PLSEntry;
    entry,entry2: PLSEntry;
BEGIN
  ASSIGN(cpls,'C:\INDEX\CURRENT.PLS');
  RESET(cpls);
  IF (i<1) OR (i>Filesize(cpls)-2) THEN BEGIN
    CLOSE(cpls);
    EXIT;
  END;
  SEEK(cpls,i);
  READ(cpls,entry);
  READ(cpls,entry2);
  SEEK(cpls,i);
  WRITE(cpls,entry2);
  WRITE(cpls,entry);
  SEEK(cpls,0);
  READ(cpls,entry);
  entry.artist:='*';
  SEEK(cpls,0);
  WRITE(cpls,entry);
  CLOSE(cpls);
END;

FUNCTION PlaylistGetTrack(num: LONGINT; VAR title,artist: STRING): BOOLEAN;
(* Aus der aktuellen Playliste den Title num zurckgeben *)
VAR cpls: FILE OF PLSEntry;
    entry: PLSEntry;
    found: BOOLEAN;
    fp,zae: LONGINT;
    ndx: FILE OF GlobalIndexRec;
    ndxentry: GlobalIndexRec;
BEGIN
  ASSIGN(cpls,'C:\INDEX\CURRENT.PLS');
  RESET(cpls);
  IF FileSize(cpls)>1 THEN BEGIN
    (* Da sind wirklich Titel drin *)
    IF FileSize(cpls)>num THEN BEGIN
      SEEK(cpls,num);
      READ(cpls,entry);
      title:=entry.title;
      artist:=entry.artist;
      PlaylistGetTrack:=TRUE;
    END ELSE BEGIN
      title:='';
      artist:='';
      PlaylistGetTrack:=FALSE;
    END;
  END ELSE BEGIN
    ASSIGN(ndx,'C:\INDEX\TITLE.IDX');
    RESET(ndx);
    IF FileSize(ndx)>=num THEN BEGIN
      SEEK(ndx,num-1);
      READ(ndx,ndxentry);
      CLOSE(ndx);
      title:=ndxentry.data.title;
      artist:=ndxentry.data.artist;
      PlaylistGetTrack:=TRUE;
    END ELSE BEGIN
      artist:='';
      title:='';
      PlaylistGetTrack:=FALSE;
    END;
  END;
  CLOSE(cpls);
END;

FUNCTION PlaylistGetTrackCount: LONGINT;
(* Aus der aktuellen Playliste die Titelzahl zurckgeben *)
VAR cpls: FILE OF PLSEntry;
    ndx: FILE OF GlobalIndexRec;
BEGIN
  ASSIGN(cpls,'C:\INDEX\CURRENT.PLS');
  RESET(cpls);
  IF FileSize(cpls)>1 THEN BEGIN
    (* Da sind wirklich Titel drin *)
    PlaylistGetTrackCount:=FileSize(cpls)-1;
  END ELSE BEGIN
    ASSIGN(ndx,'C:\INDEX\TITLE.IDX');
    RESET(ndx);
    PlaylistGetTRackCount:=FileSize(ndx);
    CLOSE(ndx);
  END;
  CLOSE(cpls);
END;

FUNCTION PlaylistEmpty: BOOLEAN;
(* Aktuelle Playlist leer? *)
VAR cpls: FILE OF PLSEntry;
BEGIN
  ASSIGN(cpls,'C:\INDEX\CURRENT.PLS');
  RESET(cpls);
  PlaylistEmpty:=(FileSize(cpls)<=1);
  CLOSE(cpls);
END;


FUNCTION PlaylistGetName: STRING;
VAR cpls: FILE OF PLSEntry;
    entry: PLSEntry;
BEGIN
  ASSIGN(cpls,'C:\INDEX\CURRENT.PLS');
  RESET(cpls);
  READ(cpls,entry);
  CLOSE(cpls);
  PlaylistGetName:=entry.title;
END;

PROCEDURE NetworkSync;
(* Synchronisation des Verz. C:\MP3 mit Netzlaufwerk W (verbunden mit MP3PUMP) *)
TYPE Databuf= ARRAY[0..32767] OF BYTE;
VAR sr: SearchRec;
    f1,f2: FILE;
    del,cp: boolean;
    buf: ^Databuf;
    br: WORD;
    bcnt: LONGINT;
    fsize: LONGINT;
    tmpstr,tit,art,com: STRING;
    proz: WORD;
    cl: BYTE;
BEGIN
  UI_CLS;
  UI_SetColors(ui_clWhite,ui_clBlack);
  UI_Write('Netzwerk-Synchronisation');
  UI_GOTOXY(1,3);
  UI_SetColors(ui_clYellow,ui_clBlack);
  UI_Write('Dateien lschen:');
  UI_CursorOff;
  UI_SetColors(ui_clCyan,ui_clBlack);
  (* berflssige Dateien lschen *)
  FindFirst('C:\MP3\*.MP3',Archive,sr);
  cl:=5;
  WHILE DosError=0 DO BEGIN
    UI_GOTOXY(1,cl);
    tmpstr:=sr.name;
    WHILE LENGTH(tmpstr)<12 DO tmpstr:=tmpstr+' ';
    UI_WRITE('Prfe "'+tmpstr+'"...');
    del:=NOT FileExists(NetSyncDrive+sr.name);
    Assign(f1,'C:\MP3\'+sr.name);
    IF NOT del THEN BEGIN
      Assign(f2,NetSyncDrive+sr.name);
      FileMode:=0;
      RESET(f2,1);
      FileMode:=2;
      RESET(f1,1);
      IF FileSize(f1)<>FileSize(f2) THEN del:=TRUE;
      CLOSE(f1);
      CLOSE(f2);
    END;
    IF del THEN BEGIN
      UI_GOTOXY(1,cl);
      UI_WRITE('Lsche "'+tmpstr+'"  ');
      Erase(f1);
      DELAY(200);
    END;
    INC(cl);
    IF cl>ui_RowCount THEN BEGIN
      cl:=5;
      UI_CLS;
      UI_SetColors(ui_clWhite,ui_clBlack);
      UI_Write('Netzwerk-Synchronisation');
      UI_GOTOXY(1,3);
      UI_SetColors(ui_clYellow,ui_clBlack);
      UI_Write('Dateien lschen:');
      UI_CursorOff;
      UI_SetColors(ui_clCyan,ui_clBlack);
    END;
    FindNext(sr);
  END;
  (* Dateien Kopieren *)
  UI_CLS;
  UI_SetColors(ui_clWhite,ui_clBlack);
  UI_Write('Netzwerk-Synchronisation');
  UI_GOTOXY(1,3);
  UI_SetColors(ui_clGreen,ui_clBlack);
  UI_Write('Kopieren: ');
  UI_CursorOff;
  UI_SetColors(ui_CLWhite,ui_clBlue);
  for proz:=5 TO ui_RowCount do begin
    UI_GOTOXY(1,proz);
    UI_WRITE('                        ');
  END;
  UI_Gotoxy(2,12);
  UI_Write('Fortschritt:');

  FindFirst(NetSyncDrive+'*.MP3',Archive,sr);
  WHILE DosError=0 DO BEGIN
    cp:=NOT FileExists('C:\MP3\'+sr.name);
    IF NOT cp THEN BEGIN
      Assign(f1,'C:\MP3\'+sr.name);
      Assign(f2,NetSyncDrive+sr.name);
      FileMode:=0;
      RESET(f2,1);
      FileMode:=2;
      RESET(f1,1);
      IF FileSize(f1)<>FileSize(f2) THEN cp:=TRUE;
      CLOSE(f1);
      CLOSE(f2);
    END;
    IF cp THEN BEGIN
      (* Kopieren *)
      GetID3Info(NetSyncDrive+sr.name,tit,art,com);
      bcnt:=0;
      New(Buf);
      Assign(f1,'C:\MP3\'+sr.name);
      Assign(f2,NetSyncDrive+sr.name);
      FileMode:=0;
      RESET(f2,1);
      FileMode:=2;
      REWRITE(f1,1);
      (* Info ausgeben *)
      UI_SetColors(ui_clYellow,ui_ClBlue);
      UI_Gotoxy(1,6);
      tmpstr:=sr.name;
      WHILE LENGTH(tmpstr)<12 DO tmpstr:=tmpstr+' ';
      UI_Write('Datei: '+tmpstr+'     ');
      UI_SetColors(ui_clWhite,ui_ClBlue);
      UI_Gotoxy(1,7);
      tmpstr:='Gre: '+IntToStr(FileSize(f2) DIV 1024)+'k';
      WHILE LENGTH(tmpstr)<24 DO tmpstr:=tmpstr+' ';
      UI_Write(tmpstr);
      (* ID3-Info ausgeben *)
      WHILE LENGTH(art)<24 DO art:=art+' ';
      art:=COPY(art,1,24);
      WHILE LENGTH(tit)<24 DO tit:=tit+' ';
      tit:=COPY(tit,1,24);
      UI_SetColors(ui_ClRed,ui_clBlue);
      UI_Gotoxy(1,9);
      UI_Write(art);
      UI_SetColors(ui_ClGreen,ui_ClBlue);
      UI_Gotoxy(1,10);
      UI_Write(tit);
      proz:=0;
      IF (DiskFree(3)-FileSize(f2))<MinFreeHDSpace THEN BEGIN
        UI_GOTOXY(2,13);
        UI_SetColors(ui_clRed,ui_clBlue);
        UI_WRITE('Nicht genug Speicher!   ');
        DELAY(2500);
        Close(f1);
        Close(f2);
        Erase(f1);
      END ELSE BEGIN
        UI_ProcBar(proz,13,ui_clGreen,ui_ClRed);
        fsize:=FileSize(f2) DIV 32768;
        bcnt:=0;
        REPEAT
          BlockRead(f2,Buf^,32768,br);
          BlockWrite(f1,Buf^,br);
          INC(bcnt);
          IF (((bcnt*100) DIV fsize)-proz)>=1 THEN BEGIN
            proz:=((bcnt*100) DIV fsize);
            UI_ProcBar(proz,13,ui_clGreen,ui_clRed);
          END;
        UNTIL br<32768;
        CLOSE(f1);
        CLOSE(f2);
      END;
      Dispose(Buf);
    END;
    FindNext(sr);
  END;
  UI_SetColors(ui_ClWhite,ui_ClBackground);
  UI_CLS;
  UI_GOTOXY(1,1);
  UI_Write('Freier Speicher auf');
  UI_GOTOXY(1,2);
  UI_Write('interner Festplatte:');
  UI_GOTOXY(1,4);
  UI_SetColors(ui_clYellow,ui_ClBackground);
  UI_Write(IntToStr(DiskFree(3))+' Bytes.');
  DELAY(2500);
  (* Das wars fast *)
  ReScanDrive(0);
END;

PROCEDURE CreateCompleteIndex(path,dn: STRING);
(* Alle indexdateien *.IDA in PATH zusammensammeln.
   sortieren und ausgeben *)
VAR ndx: IndexFile;
    idf: TEXT;
    ir,ir2:  IndexRecord;
    s:   STRING;
    swapped: BOOLEAN;
    count,zae: INTEGER;
    error: BOOLEAN;
    sr: SearchRec;
    globalentry: IndexRecord;
    ti: TEXT;
    t,a,b,q: string;
    al,ar,fz: longint;

BEGIN
  IF path[length(path)]<>'\' THEN path:=path+'\';
  ASSIGN(ndx,path+'GESAMT.TMP');
  REWRITE(ndx);
  count:=0;
  FindFirst(path+'*.IDA',Archive,sr);
  WHILE DosError=0 DO BEGIN
    ASSIGN(idf,path+sr.name);
    RESET(idf);
    WHILE NOT EOF(idf) DO BEGIN
      READLN(idf,s);
      IF s='--' THEN BEGIN
        READLN(idf,s); (* Titel *)
        error:=EOF(idf);
        IF LENGTH(s)>50 THEN s[0]:=#50;
        ir.Title:=s;
        READLN(idf,s); (* Interpret *)
        error:=error OR EOF(idf);
        IF LENGTH(s)>50 THEN s[0]:=#50;
        ir.Artist:=s;
        READLN(idf,s); (* Joliet-Pfad *)
        READLN(idf,s); (* ISO-Pfad *)
        error:=error OR EOF(idf);
        IF LENGTH(s)>100 THEN s[0]:=#100;
        ir.Path:=s;
        READLN(idf,s); (* Quality *)
        error:=error OR EOF(idf);
        ir.Quality:=s[1];
        READLN(idf,s); (* Bemerkungen *)
        IF LENGTH(s)>80 THEN s[0]:=#80;
        ir.Comment:=s;
        IF NOT error THEN BEGIN
          WRITE(ndx,ir);
          INC(count);
        END;
      END;
    END;
    CLOSE(idf);
    FindNext(sr);
  END;
  (* So, Index liegt als Datenbank vor *)
  (* Sortieren... *)
  (* Quicksort *)
  QuickSortArtist(ndx,0,FileSize(ndx)-1);
  SEEK(ndx,0);
  IF FileSize(ndx)>0 THEN READ(ndx,ir);
  al:=0; ar:=0;
  FOR fz:=1 TO FileSize(ndx)-1 DO BEGIN
    SEEK(ndx,fz);
    READ(ndx,ir2);
    IF MakeCompareable(ir.artist)<>MakeCompareable(ir2.artist) THEN BEGIN
      QuickSortTitle(ndx,al,ar);
      al:=fz; ar:=fz;
      ir:=ir2;
    END ELSE BEGIN
      ar:=fz;
    END;
  END;
  QuickSortTitle(ndx,al,ar);

  SEEK(ndx,0);
  ASSIGN(ti,dn);
  REWRITE(ti);
  WRITELN(ti,'Gesamtindex der vorhandenen Titel:');
  WRITELN(ti,'==================================');
  WRITELN(ti,'');
  WRITELN(ti,'Qualittsstufen:');
  WRITELN(ti,' A: von CD, Datenrate > 128kBit/s');
  WRITELN(ti,' B: von CD, Datenrate = 128kBit/s');
  WRITELN(ti,' C: von CD, Datenrate 112kBit/s oder Layer II codiert');
  WRITELN(ti,' D: z.B. aus Radio/TV; sehr gute Qualitt oder < 44kHz');
  WRITELN(ti,' E: aus Radio/TV; gute Qualitt');
  WRITELN(ti,' F: aus Radio/TV; befriedigende Qualitt');
  WRITELN(ti,' G: z.B. von Cassette; mige Qualitt');
  WRITELN(ti,' H: geringe Qualitt');
  WRITELN(ti,'');
  WRITELN(ti,FileSize(ndx),' Titel insgesamt.');
  WRITELN(ti,'');
  WRITELN(ti,'Interpret                     Titel                         Qly Bemerkungen');
  WRITELN(ti,'-------------------------------------------------------------------------------');
  WHILE NOT EOF(ndx) DO BEGIN
    READ(ndx,globalentry);
    t:=globalentry.title;
    a:=globalentry.artist;
    b:=globalentry.comment;
    q:=' '+globalentry.quality+'  ';
    IF LENGTH(t)>29 THEN t:=COPY(t,1,29);
    WHILE LENGTH(t)<30 DO t:=t+' ';
    IF LENGTH(a)>29 THEN a:=COPY(a,1,29);
    WHILE LENGTH(a)<30 DO a:=a+' ';
    IF LENGTH(b)>15 THEN b:=COPY(b,1,15);
    WRITELN(ti,a+t+q+b);
  END;
  CLOSE(ti);
  CLOSE(ndx);
  ERASE(ndx);
END;

PROCEDURE PLSIndicatorADD(plsi: PPLSIndicator; num: WORD);
BEGIN
  plsi^[num DIV 8]:=
    plsi^[num DIV 8] OR (1 SHL (num MOD 8));
END;
PROCEDURE PLSIndicatorDEL(plsi: PPLSIndicator; num: WORD);
BEGIN
  plsi^[num DIV 8]:=
    plsi^[num DIV 8] AND (255-(1 SHL (num MOD 8)));
END;

PROCEDURE RecreatePLSIndicators;
(* Indikatoren, ob Titel in aktueller Playliste ist, neu erstellen *)
VAR zae: LONGINT;
    cnt: LONGINT;
    tmp: LONGINT;
    tit,art: STRING;
BEGIN
  FOR zae:=0 TO 8191 DO BEGIN
    PLSArtistIndicator^[zae]:=0;
    PLSTitleIndicator^[zae]:=0;
  END;
  IF PlaylistEmpty THEN EXIT;
  cnt:=PlaylistGetTrackCount;
  FOR zae:=1 TO cnt DO BEGIN
    PlaylistGetTrack(zae,tit,art);
    tmp:=GetTrackArtistIndexNum(tit,art);
    IF tmp>=0 THEN PLSIndicatorADD(PLSArtistIndicator,tmp);
    tmp:=GetTrackTitleIndexNum(tit,art);
    IF tmp>=0 THEN PLSIndicatorADD(PLSTitleIndicator,tmp);
  END;
END;

FUNCTION InPLSbyArtist(num: LONGINT): BOOLEAN;
BEGIN
  IF (num>=0) AND (num<=65535) THEN BEGIN
    InPLSbyArtist:=
      (PLSArtistIndicator^[num DIV 8] AND (1 SHL (num MOD 8)))<>0;
  END ELSE
    InPLSbyArtist:=FALSE;
END;

FUNCTION InPLSbyTitle(num: LONGINT): BOOLEAN;
BEGIN
  IF (num>=0) AND (num<=65535) THEN BEGIN
    InPLSbyTitle:=
      (PLSTitleIndicator^[num DIV 8] AND (1 SHL (num MOD 8)))<>0;
  END ELSE
    InPLSbyTitle:=FALSE;
END;

BEGIN
  New(PLSArtistIndicator);
  New(PLSTitleIndicator);
  RecreatePLSIndicators;
END.

