UNIT MP3Pump;
(* Steuerroutinen fr den PUMP *)


INTERFACE
PROCEDURE InitMAS;
PROCEDURE ExitMas;
PROCEDURE DataCheck;
PROCEDURE OpenFile(dn: STRING);
PROCEDURE Play;
PROCEDURE Pause;
PROCEDURE Stop;
PROCEDURE Skip_FF;
PROCEDURE Skip_REW;
PROCEDURE VolumeUp;
PROCEDURE VolumeDown;
FUNCTION AllDataDone: BOOLEAN;
PROCEDURE GetStreamInfo(VAR Bitrate,Samplerate: WORD; Var mpeg,layer: BYTE);
PROCEDURE GetPosition(VAR min,sec: WORD);
PROCEDURE GetPositionLeft(VAR min,sec: WORD);

IMPLEMENTATION

USES MP3PLS,DOS,PUMPi2C,crt,mp3uicom;

{$DEfine FastOutput}

CONST cBufSize = 4096; (* mu n*32 sein!!! *)
      cBufNum = 40;
      cMinFilledBuf = 1;
      LPT_Data = $378;
      LPT_IRQ=7;
      LPT_Ctrl = LPT_Data+2;
      LPT_Stat = LPT_Data+1;
      LPT_ACK = $40;
      LPT_EACK = $10;
      LPT_STROBE = $1;
      LPT_AUTOLF = $2;
      LPT_INIT = $4;
      LPT_SELECT = $8;
      LPT_BIDIR = $20;
      LPT_ERROR = $8;
      LPT_SELIN = $10;
      LPT_PAPER = $20;
      LPT_BUSY = $80;
      INT_MasterCtrl = $20;
      INT_EOI = $20;
      WR_MP3 = 0;
      WR_INP_SEL = 1;
      WR_LCD = 3;
      RD_INP_STAT = 0;
      RD_INP_VAL = 1;
      RD_LCD_BUSY = 2;
      GET_KEY_INP = $12;
      GET_IR_DATA = $10;
      GET_IR_ADR = $11;
      GET_KNOP_INP = $14;
      RD_STAT_MASK = $3c;
      ATMEL_REG0 = 0;
      ATMEL_REG1 = 1;
      DISPLAY_REG0 = 2;
      DISPLAY_REG1 = 3;
      ATMEL_MAS = 0;
      ATMEL_STATUS = 0;
      ATMEL_CMD = 1;
      DISPLAY_CMD = 2;
      DISPLAY_DATA = 3;
      TMPBUF0 = 4;
      TMPBUF1 = 5;
      WRITE_0 = 0;
      WRITE_1 = 1;
      WRITE_2 = 2;
      WRITE_3 = 3;
      WRITE_4 = 4;
      WRITE_5 = 5;

TYPE TMP3Buf = RECORD
       data: ARRAY[0..cBufSize-1] OF BYTE;
       done: BOOLEAN;
       count: WORD;
     END;
     PMP3Buf = ^TMP3Buf;

VAR Buffer: ARRAY[1..cBufNum] OF PMP3Buf;
    CurReadBuf, CurWriteBuf, CurReadPos: WORD;
    FileOpened: BOOLEAN;
    CurFile: FILE;
    paused,broken: BOOLEAN;
    SavedISR: POINTER;
    BytesPlayed,FileBytes: LONGINT;
    CurFile_BitRate: WORD;
    CurFile_SampleRate: WORD;
    CurFile_MPEG: BYTE;
    CurFile_Layer: BYTE;
    CurVolume: LONGINT;

PROCEDURE MasIRQ; INTERRUPT; FORWARD;
PROCEDURE DisableISR; FORWARD;
PROCEDURE ReadStreamInfo(VAR Bitrate,Samplerate: WORD; Var mpeg,layer: BYTE); FORWARD;

PROCEDURE RegWrite(reg,data: BYTE);
CONST strg: ARRAY[0..5] OF BYTE =($1a,$12,$18,$10,$1d,$15);
VAR tot: WORD;
BEGIN
  (*Schreibzugriff mit Strobeflag=0 ankuendigen *)
  port[LPT_Ctrl]:=strg[reg];
  (* Daten parallel anlegen *)
  port[LPT_Data]:=data;
  (* Daten mit Strobeflag=1 gueltig setzen *)
  port[LPT_Ctrl]:=strg[reg]+LPT_Strobe;
  {$IFNDEF FastOutput}
  tot:=0;
  (* Warten auf Busyflag=high innerhalb Timeout *)
  WHILE ((PORT[LPT_Stat] AND LPT_BUSY)=0) AND (tot<1000) DO INC(tot);
  PORT[LPT_CTRL]:=strg[reg];
  tot:=0;
  (* Warten auf Busyflag=low innerhalb Timeout *)
  WHILE ((PORT[LPT_Stat] AND LPT_BUSY)<>0) AND (tot<1000) DO INC(tot);
  {$ENDIF}
  asm sti; end;
END;

PROCEDURE InitMAS;
(* Buffer Init; Int-Init *)
VAR zae: WORD;
BEGIN
  IF console then exit;
  FOR zae:=1 TO cBufNum DO BEGIN
    New(Buffer[zae]);
    Buffer[zae]^.done:=TRUE;
    Buffer[zae]^.count:=0;
  END;
  CurReadBuf:=1;
  CurWriteBuf:=1;
  CurReadPos:=0;
  FileOpened:=FALSE;
  Paused:=TRUE;
  broken:=TRUE;
  RegWrite(ATMEL_CMD,$4) ;(* RESET*)
  Send_DWord($3a,$68,$9e62,$0); (* Divider ein *)
  Send_Word($3A,$68,$0fcd);
  Write_D0($32D,$2AAAB);
  Write_D0($32E,$A0001);
  Send_Word($3A,$68,$0fcb);
  CurVolume:=$80000;
  Write_D1($7F8,CurVolume);
  Write_D1($7FB,CurVolume);
  GetIntVec(LPT_IRQ+8,SavedISR);
  SetIntVec(LPT_IRQ+8,Addr(MASIRQ));
END;

PROCEDURE ExitMAS;
(* Speicher freigeben *)
VAR zae: WORD;
BEGIN
  IF console then exit;
  DisableISR;
  SetIntVec(LPT_IRQ+8,SavedISR);
  FOR zae:=1 TO cBufNum DO BEGIN
    Dispose(Buffer[zae]);
  END;
END;

PROCEDURE EnableISR;
BEGIN
  PORT[INT_MasterCtrl+1]:=PORT[INT_MasterCtrl+1] AND ($FF-(1 SHL LPT_IRQ));
  PORT[LPT_CTRL]:=PORT[LPT_Ctrl] OR LPT_EACK;
END;
PROCEDURE DisableISR;
BEGIN
  PORT[LPT_Ctrl]:=PORT[LPT_Ctrl] AND ($FF-LPT_EACK);
  PORT[INT_MasterCtrl+1]:=PORT[INT_MasterCtrl+1] OR (1 SHL LPT_IRQ);
END;

PROCEDURE DataCheck;
(* Prfen, ob Buffer Nachzuladen *)
VAR DataRead: WORD;
    regs: REGISTERS;
BEGIN
  IF console then exit;
  IF NOT FileOpened THEN EXIT; (* Nix da zum Spielen *)
  IF NOT Buffer[CurWriteBuf]^.done THEN EXIT; (* Noch kein leerer Buffer da *)
  BlockRead(CurFile,Buffer[CurWriteBuf]^.data,cBufSize,DataRead);
  IF DataRead<cBufSize THEN BEGIN
    (* Datei alle *)
    (* Close(CurFile);
    FileOpened:=FALSE; *)
  END;
  IF DataRead>0 THEN BEGIN
    (* Neue Daten da *)
    Buffer[CurWriteBuf]^.count:=DataRead;
    Buffer[CurWriteBuf]^.done:=FALSE;
    INC(CurWriteBuf);
    IF CurWriteBuf>cBufNum THEN CurWriteBuf:=1;
    IF broken AND NOT paused THEN BEGIN
      (* ReInit IRQ *)
      INTR(LPT_IRQ+8,regs);
    END;
  END;
END;
{$I-}{$S-}{$R-}{$Q-}
PROCEDURE MasIRQ;
(* Interrupt fr Datenbertragung *)
VAR zae: WORD;
BEGIN
  ASM
    cli;
  END;
  broken:=FALSE;
  IF Buffer[CurReadBuf]^.done THEN broken:=TRUE ELSE
    IF paused THEN broken:=TRUE;
  IF NOT broken THEN BEGIN
    (* Bytes bertragen *)
    WHILE ((port[LPT_Stat] AND LPT_Ack)<>0) AND NOT broken DO BEGIN
      (* Sende Byte an MAS *)
      {$IFDEF FastOutput}
        (*Schreibzugriff mit Strobeflag=0 ankuendigen *)
        port[LPT_Ctrl]:=$1a;
        (* Daten parallel anlegen *)
        port[LPT_Data]:=Buffer[CurReadBuf]^.data[CurReadPos];
        (* Daten mit Strobeflag=1 gueltig setzen *)
        port[LPT_Ctrl]:=$1a+LPT_Strobe;
      {$ELSE}
        RegWrite(WR_MP3,Buffer[CurReadBuf]^.data[CurReadPos]);
      {$ENDIF}
      INC(BytesPlayed);
      INC(CurReadPos);
      IF Buffer[CurReadBuf]^.count<=CurReadPos THEN BEGIN
        (* DIeser Buffer leer *)
        Buffer[CurReadBuf]^.done:=TRUE;
        INC(CurReadBuf);
        CurReadPos:=0;
        IF CurReadBuf>cBufNum THEN CurReadBuf:=1;
        IF Buffer[CurReadBuf]^.done THEN broken:=TRUE;
      END;
    END;
  END;
  (* EOI Signal *)
  PORT[INT_MASTERCTRL]:=INT_EOI;
  ASM
    sti;
  END;
END;

PROCEDURE OpenFile(dn: STRING);
(* Datei zum Abspielen ffnen *)
VAR zae,br: WORD;
    tempBuf: ARRAY[0..2999] OF BYTE;
    t1,t2: word; t3,t4: BYTE;
BEGIN
  IF console then exit;
  IF FileOpened THEN Close(CurFile);
  IF NOT FIleExists(dn) THEN BEGIN
    Stop;
    EXit;
  END;
  DisableISR;
  ASSIGN(CurFile,dn);
  FileMode:=0;
  RESET(CurFile,1);
  FileMode:=2;
  FileOpened:=TRUE;
  (* So, jetzt ein paar Hundert Bytes an den MAS senden
     und die MP3-Daten rauskriegen *)
  BlockRead(CurFile,tempBuf,3000,br);
  Send_DWord($3a,$68,$9aa1,$0); (* Mute *)
  FOR zae:=0 to br-1 DO BEGIN
    RegWrite(WR_MP3,tempBuf[zae]);
    WHILE ((port[LPT_Stat] AND LPT_Ack)=0) DO;
  END;
  (* Jetzt Daten suchen *)
  FOR zae:=0 to 3000 DO BEGIN
    RegWrite(WR_MP3,0);
    WHILE ((port[LPT_Stat] AND LPT_Ack)=0) DO;
  END;
  Delay(10);
  REPEAT
    ReadStreamInfo(t1,t2,t3,t4);
    ReadStreamInfo(CurFile_BitRate,CurFile_SampleRate,CurFile_MPEG,CurFile_Layer);
  UNTIL (CurFile_BitRate=t1) AND (CurFile_SampleRate=t2) AND (CurFile_MPEG=t3) AND (CurFile_Layer=t4);
  Send_DWord($3a,$68,$9aa0,$0); (* Un-Mute *)
  Send_DWord($3a,$68,$9aa0,$0); (* Un-Mute *)
  SEEK(CurFIle,0);
  FileBytes:=FileSize(CurFile);
  (* fertig *)
  FOR zae:=1 To cBufNum DO DataCheck; (* Buffer fllen *)
  BytesPlayed:=0;
END;

PROCEDURE Play;
(* Abspielen starten *)
VAR regs: registers;
BEGIN
  IF console then exit;
  if not paused and not broken then exit;
  paused:=FALSE;
  EnableISR;
  Intr(LPT_IRQ+8,regs);
  broken:=FALSE;
END;
PROCEDURE Pause;
(* Abspielen anhalten *)
BEGIN
  IF console then exit;
  if paused then exit;
  paused:=TRUE;
  DisableISR;
END;
PROCEDURE Stop;
(* Stck abbrechen *)
VAR zae: WORD;
BEGIN
  IF console then exit;
  DisableISR;
  paused:=TRUE;
  FOR zae:=1 TO cBufNum DO
    Buffer[zae]^.done:=TRUE;
  CurReadPos:=0;
  CurReadBuf:=1;
  CurWriteBuf:=1;
  IF FileOpened THEN Close(CurFile);
  FileOpened:=FALSE;
  broken:=TRUE;
END;
PROCEDURE Skip_FF;
(* Suchlauf vorwrts *)
var zae: byte;
BEGIN
  IF console then exit;
  IF NOT paused THEN BEGIN
    ASM cli; END;
    for zae:=1 to 10 do begin
      if NOT Buffer[CurReadBuf]^.done THEN BEGIN
        BytesPlayed:=BytesPlayed+cBufSize;
        Buffer[CurReadBuf]^.done:=TRUE;
        INC(CurReadBuf);
        IF CurReadBuf>cBufNum THEN CurReadBuf:=1;
      END;
    end;
    BytesPlayed:=BytesPlayed-CurReadPos;
    CurReadPos:=0;
    ASM sti; end;
  END;
END;
PROCEDURE Skip_REW;
(* Suchlauf rckwrts *)
VAR zae: WORD;
    np: LONGINT;
    regs: REGISTERS;
BEGIN
  IF console then exit;
  IF paused THEN EXIT;
  DisableISR;
  (* Alle Buffer tten *)
  FOR zae:=1 TO cBufNum DO
    Buffer[zae]^.done:=TRUE;
  CurReadPos:=0;
  CurReadBuf:=1;
  CurWriteBuf:=1;
  np:=BytesPlayed-14*cBufSize;
  IF np<0 THEN np:=0;
  BytesPlayed:=np;
  SEEK(CurFile,np);
  FOR zae:=1 TO cMinFilledBuf DO DataCheck; (* Buffer fllen *)
  EnableISR;
  Intr(LPT_IRQ+8,regs);
  broken:=FALSE;
END;

FUNCTION FileDone: BOOLEAN;
BEGIN
  IF NOT FileOpened THEN BEGIN
    FileDone:=TRUE;
    EXIT;
  END ELSE BEGIN
    FileDone:=(FileSize(CurFile)=FilePos(CurFile));
  END;
END;
FUNCTION AllDataDone: BOOLEAN;
BEGIN
  AllDataDone:=FileDone AND broken;
END;

PROCEDURE ReadStreamInfo(VAR Bitrate,Samplerate: WORD; Var mpeg,layer: BYTE);
(* via IC MP3-Stream-Info auslesen *)
VAR tmp: longint;
    sr,br: word;
CONST BitRates_1L3: ARRAY[0..15] OF WORD = (0,32,40,48,56,64,80,96,112,128,160,192,224,256,320,0);
      BitRates_1L2: ARRAY[0..15] OF WORD = (0,32,48,56,64,80,96,112,128,160,192,224,256,320,384,0);
      BitRates_2Lx: ARRAY[0..15] OF WORD = (0,8,16,24,32,40,48,56,64,80,96,112,128,144,160,0);
      SampleRates_1: ARRAY[0..3] OF WORD = (44100,48000,32000,0);
      SampleRates_2: ARRAY[0..3] OF WORD = (22050,24000,16000,0);
      SampleRates_25: ARRAY[0..3] OF WORD = (11025,12000,8000,0);

BEGIN
  IF console then exit;
  while not Read_D0($301,tmp) do;
  layer:=(tmp SHR 11) AND 3;
  layer:=4-layer;
  mpeg:=(tmp SHR 13) AND 3;
  if mpeg=3 then mpeg:=1; (* 0= 2.5, 1=1, 2=2 *)
  while not Read_D0($302,tmp) do;
  br:=(tmp SHR 12) AND 15;
  sr:=(tmp SHR 10) AND 3;
  case mpeg OF
    1: if layer=2 THEN
         BitRate:=BitRates_1L2[br]
       else
         BitRate:=BitRates_1L3[br];
    0,2: BitRate:=BitRates_2Lx[br];
  end;
  case mpeg of
    1: Samplerate:=SampleRates_1[sr];
    2: Samplerate:=SampleRates_2[sr];
    0: Samplerate:=SampleRates_25[sr];
  end;
END;

PROCEDURE GetStreamInfo(VAR Bitrate,Samplerate: WORD; Var mpeg,layer: BYTE);
BEGIN
  BitRate:=CurFile_BitRate;
  SampleRate:=CurFile_SampleRate;
  mpeg:=CurFile_MPEG;
  layer:=CurFile_Layer;
END;

PROCEDURE GetPosition(VAR min,sec: WORD);

VAR ts: LONGINT;

BEGIN
  if CurFile_BitRate<>0 THEN
    ts:=((BytesPlayed*8) DIV CurFile_BitRate) DIV 1024
  ELSE
    ts:=((BytesPlayed*8) DIV 128) DIV 1024; (* wenn nicht vorhanden *)
  min:=ts DIV 60;
  sec:=ts MOD 60;
END;

PROCEDURE GetPositionLeft(VAR min,sec: WORD);

VAR ts: LONGINT;

BEGIN
  if CurFile_BitRate<>0 THEN
    ts:=(((FileBytes-BytesPlayed)*8) DIV CurFile_BitRate) DIV 1024
  ELSE
    ts:=(((FileBytes-BytesPlayed)*8) DIV 128) DIV 1024; (* wenn nicht vorhanden *)
  IF ts<0 THEN ts:=0;
  min:=ts DIV 60;
  sec:=ts MOD 60;
END;

PROCEDURE VolumeUp;
var regs: registers;
(* Lauter *)
BEGIN
  IF console then exit;
  IF CurVolume<=$80000 THEN EXIT;
  CurVolume:=CurVolume-8192;
  Write_D1($7F8,CurVolume);
  if not paused then begin
    EnableISR;
    Intr(LPT_IRQ+8,regs);
    broken:=FALSE;
    DataCheck;
  end;
  Write_D1($7FB,CurVolume);
  if not paused then begin
    EnableISR;
    Intr(LPT_IRQ+8,regs);
    broken:=FALSE;
  end;
END;
PROCEDURE VolumeDown;
(* Leiser *)
var regs: registers;
BEGIN
  IF console then exit;
  IF CurVolume>=$FE000 THEN EXIT;
  CurVolume:=CurVolume+8192;
  Write_D1($7F8,CurVolume);
  if not paused then begin
    EnableISR;
    Intr(LPT_IRQ+8,regs);
    broken:=FALSE;
    DataCheck;
  end;
  Write_D1($7FB,CurVolume);
  if not paused then begin
    EnableISR;
    Intr(LPT_IRQ+8,regs);
    broken:=FALSE;
  end;
END;

BEGIN
  FILEBytes:=0;
END.





