UNIT MP3UICOM;
(* Routinen zur UI-Datstellung *)
INTERFACE

CONST ui_clBackground = 0;
      ui_clBlack      = 0;
      ui_clRed        = 1;
      ui_clGreen      = 2;
      ui_clBlue       = 3;
      ui_clYellow     = 4;
      ui_clCyan       = 5;
      ui_clMagenta    = 6;
      ui_clWhite      = 7;
      ui_clGray       = 8;
      ui_clLightGreen = 9;
      ui_clLightBlue  = 10;
      ui_clLightRed   = 11;
      ui_clCursor     = 255;


      ui_ColCount = 24;
      ui_RowCount = 14;
      ui_MinASCII = #32;
      ui_MaxASCII = #255;

PROCEDURE UI_CLS;
PROCEDURE UI_GOTOXY(x,y: BYTE);
PROCEDURE UI_Write(s: STRING);
PROCEDURE UI_CursorOff;
PROCEDURE UI_CursorOn;
PROCEDURE UI_SetColors(fg,bg: BYTE);
PROCEDURE UI_UnHiLite(ci: BYTE);
PROCEDURE UI_HiLite(ci: BYTE);
PROCEDURE UI_TrackInfo1(i: STRING);
PROCEDURE UI_TrackInfo2(i: STRING);
PROCEDURE UI_TrackInfo3(i: STRING);
PROCEDURE UI_TimeInfo(i: STRING; shuffle: BOOLEAN);
PROCEDURE UI_TimeInfoP(i: STRING; shuffle: BOOLEAN);
PROCEDURE UI_EnterExclusiveMode;
PROCEDURE UI_ExitExclusiveMode;
PROCEDURE UI_Ready;
FUNCTION UI_GetEvent(VAR ev: CHAR): BOOLEAN;
PROCEDURE UI_ProcBar(proz,line: BYTE; barcolor,txtcolor: BYTE);

VAR Console: BOOLEAN;

IMPLEMENTATION

USES MP3Com,CRT,MP3PLS;

VAR CommandLine: STRING;
    LastPBLine: STRING;
    LastPBCols: STRING;

FUNCTION ANSI2Z80(c: CHAR): CHAR;

VAR v: CHAR;

BEGIN
  CASE c OF
    #0..#126: v:=c;
    #145,#146: v:=#39; (* ' *)
    #173: v:='-';
    #176: v:=#127; (*  *)
    #178: v:=#131; (*  *)
    #180: v:=#39;  (* ' *)
    #192..#195,#197..#198: v:='A';
    #196: v:=#134; (*  *)
    #199: v:='C';
    #200..#203: v:='E';
    #204..#207: v:='I';
    #208: v:='D';
    #209: v:='N';
    #210..#213: v:='O';
    #214: v:=#135; (*  *)
    #215: v:='x';
    #217..#219: v:='U';
    #220: v:=#136; (**)
    #221: v:='Y';
    #223: v:=#137; (**)
    #224..#227,#229..#230: v:='a';
    #228: v:=#133; (**)
    #231: v:='c';
    #232..#235: v:='e';
    #236..#239: v:='i';
    #240: v:='d';
    #241: v:='n';
    #242..#245: v:='o';
    #246: v:=#132; (**)
    #249..#251: v:='u';
    #252: v:=#130; (**)
    #253,#255: v:='y';
    ELSE v:='_';
  END;
  Ansi2Z80:=v;
END;

PROCEDURE AZ8S(VAR s: STRING);
VAR zae: BYTE;
BEGIN
  FOR zae:=1 TO LENGTH(s) DO s[zae]:=Ansi2Z80(s[zae]);
END;

FUNCTION Hex2Byte(s: string): BYTE;
VAR t: BYTE;
BEGIN
  IF LENGTH(s)<2 THEN EXIT;
  t:=0;
  s[2]:=UpCase(s[2]);
  CASE s[2] OF
    '0'..'9': t:=t+(ORD(s[2])-48);
    'A'..'F': t:=t+(ORD(s[2])-55);
  END;
  s[1]:=UpCase(s[1]);
  CASE s[1] OF
    '0'..'9': t:=t+16*(ORD(s[1])-48);
    'A'..'F': t:=t+16*(ORD(s[1])-55);
  END;
  Hex2Byte:=t;
END;

FUNCTION Byte2Hex(b: BYTE): STRING;
VAR s: STRING;
BEGIN
  IF (b DIV 16)<10 THEN
    s:=CHR(48+(b DIV 16))
  ELSE
    s:=CHR(55+(b DIV 16));
  IF (b MOD 16)<10 THEN
    s:=s+CHR(48+(b MOD 16))
  ELSE
    s:=s+CHR(55+(b MOD 16));
  Byte2Hex:=s;
END;
PROCEDURE UI_CLS;
BEGIN
  IF console THEN ClrScr ELSE V24WriteString('Y0'#13);
END;

PROCEDURE UI_GOTOXY(x,y: BYTE);
BEGIN
  IF Console THEN GOTOXY(x,y) ELSE V24WriteString('Y1'+CHR(65+x)+CHR(65+y)+#13);
END;

PROCEDURE UI_Write(s: STRING);
BEGIN
  IF Console THEN Write(s) ELSE BEGIN
    AZ8S(s);
    V24WriteString('Y2'+s+#13);
  END;
END;

PROCEDURE UI_CursorOff;
BEGIN
  IF NOT Console THEN V24WriteString('Y4'+#13);
END;
PROCEDURE UI_CursorOn;
BEGIN
  IF NOT Console THEN V24WriteString('Y3'+#13);
END;

PROCEDURE UI_SetColors(fg,bg: BYTE);

BEGIN
  IF NOT Console THEN V24WriteString('Y5'+Byte2Hex(fg)+Byte2Hex(bg)+#13);
END;

PROCEDURE UI_UnHiLite(ci: BYTE);
BEGIN
  IF NOT Console THEN V24WriteString('Y7'+chr(ci+65)+#13);
END;

PROCEDURE UI_HiLite(ci: BYTE);
BEGIN
  IF NOT Console THEN
    V24WriteString('Y6'+chr(ci+65)+#13)
  ELSE BEGIN
    gotoxy(1,25); write(ci);
  END;
END;

PROCEDURE UI_TrackInfo1(i: STRING);
BEGIN
  AZ8S(i);
  IF LENGTH(i)>24 THEN i:=copy(i,1,24);
  IF NOT Console THEN
    V24WriteString('A'+i+#13)
  ELSE BEGIN
    Gotoxy(1,20); Write(i);
  END;
END;
PROCEDURE UI_TrackInfo2(i: STRING);
BEGIN
  AZ8S(i);
  IF LENGTH(i)>24 THEN i:=copy(i,1,24);
  IF NOT Console THEN
    V24WriteString('T'+i+#13)
  ELSE BEGIN
    Gotoxy(1,21); Write(i);
  END;
END;
PROCEDURE UI_TrackInfo3(i: STRING);
BEGIN
  AZ8S(i);
  IF LENGTH(i)>6 THEN i:=copy(i,1,6);
  IF NOT Console THEN
    V24WriteString('I'+i+#13)
  ELSE BEGIN
    Gotoxy(10,22); Write(i);
  END;
END;
PROCEDURE UI_TimeInfo(i: STRING; shuffle: BOOLEAN);
BEGIN
  AZ8S(i);
  IF LENGTH(i)>5 THEN i:=COPY(i,1,5);
  IF NOT Console THEN
    IF shuffle THEN
      V24WriteString('Z'+i+#13+'QS'+#13)
    ELSE
      V24WriteString('Z'+i+#13+'Q_'+#13)
  ELSE BEGIN
    Gotoxy(1,22); Write(i);
  END;
END;
PROCEDURE UI_TimeInfoP(i: STRING; shuffle: BOOLEAN);
BEGIN
  AZ8S(i);
  WHILE LENGTH(i)<5 DO i:=i+' ';
  IF NOT Console THEN
    IF shuffle THEN
      V24WriteString('Z'+i+#13+'PS'+#13)
    ELSE
      V24WriteString('Z'+i+#13+'P_'+#13);
END;

PROCEDURE UI_EnterExclusiveMode;
BEGIN
  (* Dies geschieht mittlerweile Automatisch *)
  IF Console THEN ClrScr;
END;
PROCEDURE UI_ExitExclusiveMode;
BEGIN
  IF NOT Console THEN V24WriteString('X'+#13+'X'+#13);
END;

PROCEDURE UI_Ready;
BEGIN
  IF NOT COnsole THEN V24WriteString('R'+#13+'R'+#13);
END;

FUNCTION UI_GetEvent(VAR ev: CHAR): BOOLEAN;
VAR tmp: string;
BEGIN
  IF Console THEN BEGIN
    if keypressed THEN BEGIN
      ev:=READKEY;
      UI_GetEvent:=true;
    END ELSE BEGIN
      UI_GetEvent:=false;
    end;
    EXIT;
  END;
  ev:=#00;
  V24DataCheck;
  tmp:=V24ReadString;
  WHILE tmp<>'' DO BEGIN
    IF tmp<>#13 THEN
      CommandLine:=CommandLine+tmp
    ELSE BEGIN
      IF LENGTH(CommandLine)>2 THEN BEGIN
        IF CommandLine[1]='K' THEN BEGIN
          (* Ein Tastenereignis *)
          ev:=CHR(Hex2Byte(COPY(CommandLine,2,2)));
        END;
      END;
      CommandLine:='';
    END;
    V24DataCheck;
    tmp:=V24ReadString;
  END;
  UI_GetEvent:=(ev<>#00);
END;

PROCEDURE UI_ProcBar(proz,line: BYTE; barcolor,txtcolor: BYTE);
VAR colswpos: WORD;
    bt,ct: STRING;
    curpos,curcol: BYTE;
    zae: BYTE;

BEGIN
  IF proz=0 THEN BEGIN
    LastPBLine:='************************';
  END;
  IF NOT Console THEN BEGIN
    bt:=IntToStr(proz)+'%';
    IF proz<10 THEN bt:=' '+bt+' ' ELSE IF proz<100 THEN bt:=bt+' ';
    bt:='          '+bt+'          ';
    (* Bt enthlt die Zeile *)
    (* Jetzt noch Farben Festlegen *)
    colswpos:=((22*proz) DIV 100)+2;
    FOR zae:=2 TO 23 DO BEGIN
      IF zae<colswpos THEN ct[zae]:=CHR(barcolor)
                      ELSE ct[zae]:=CHR(ui_clGray);
    END;
    ct[1]:=CHR(ui_clBlue);
    ct[24]:=CHR(ui_clBlue);
    (* Ausgabe *)
    curcol:=255;
    curpos:=255;
    FOR zae:=1 TO 24 DO BEGIN
      IF (bt[zae]<>LastPBLine[zae]) OR
         (ct[zae]<>LastPBCols[zae]) THEN BEGIN
        (* Diese Stelle anzeigen *)
        IF curcol<>ORD(ct[zae]) THEN BEGIN
          curcol:=ORD(ct[zae]);
          UI_SetColors(txtcolor,curcol);
        END;
        IF curpos<>zae THEN BEGIN
          curpos:=zae;
          UI_GOTOXY(curpos,line);
        END;
        UI_WRITE(bt[zae]);
        INC(curpos);
      END;
    END;
    LastPBLine:=bt;
    LastPBCols:=ct;
  END;
END;
BEGIN
  CommandLine:='';
  Console:=FALSE;
END.